/*******************************************************************************
* File Name: UART_CO2_INT.c
* Version 2.0
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "UART_CO2.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (UART_CO2_RX_ENABLED || UART_CO2_HD_ENABLED) && \
     (UART_CO2_RXBUFFERSIZE > UART_CO2_FIFO_LENGTH))

    extern volatile uint8 UART_CO2_rxBuffer[];
    extern volatile uint8 UART_CO2_rxBufferRead;
    extern volatile uint8 UART_CO2_rxBufferWrite;
    extern volatile uint8 UART_CO2_rxBufferLoopDetect;
    extern volatile uint8 UART_CO2_rxBufferOverflow;


    /*******************************************************************************
    * Function Name: UART_CO2_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UART_CO2_rxBuffer - RAM buffer pointer for save received data.
    *  UART_CO2_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     increments after each byte saved to buffer.
    *  UART_CO2_rxBufferRead - cyclic index for read from rxBuffer, 
    *     checked to detect overflow condition.
    *  UART_CO2_rxBufferOverflow - software overflow flag. Set to one
    *     when UART_CO2_rxBufferWrite index overtakes 
    *     UART_CO2_rxBufferRead index.
    *  UART_CO2_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when UART_CO2_rxBufferWrite is equal to 
    *    UART_CO2_rxBufferRead
    *
    *******************************************************************************/
    CY_ISR(UART_CO2_RXISR)
    {
        uint8 readData;

        /* User code required at start of ISR */
        /* `#START UART_CO2_RXISR_START` */

        /* `#END` */

        readData = UART_CO2_RXSTATUS_REG;

        if(readData & (UART_CO2_RX_STS_BREAK | UART_CO2_RX_STS_PAR_ERROR | \
                       UART_CO2_RX_STS_STOP_ERROR | UART_CO2_RX_STS_OVERRUN))
        {
            /* ERROR handling. */
            /* `#START UART_CO2_RXISR_ERROR` */

            /* `#END` */
        }

        while(readData & UART_CO2_RX_STS_FIFO_NOTEMPTY)
        {
            /* Check pointer. */
            if(UART_CO2_rxBufferWrite >= UART_CO2_RXBUFFERSIZE)
            {
                UART_CO2_rxBufferWrite = 0u;
                if(UART_CO2_rxBufferWrite == UART_CO2_rxBufferRead)
                {   /* Set Software Buffer status Overflow */
                    UART_CO2_rxBufferOverflow = 1u;
                }
            }

            UART_CO2_rxBuffer[UART_CO2_rxBufferWrite] = UART_CO2_RXDATA_REG;

            if(UART_CO2_rxBufferLoopDetect)
            {   /* Set Software Buffer status Overflow */
                UART_CO2_rxBufferOverflow = 1u;
            }
            /* Set next pointer. */
            UART_CO2_rxBufferWrite++;

            if(UART_CO2_rxBufferWrite == UART_CO2_rxBufferRead)
            {
                UART_CO2_rxBufferLoopDetect = 1u;
            }

            /* Check again if there is data. */
            readData = UART_CO2_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START UART_CO2_RXISR_END` */

        /* `#END` */

        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(UART_CO2_PSOC3_ES2 && (UART_CO2_RXInternalInterrupt__ES2_PATCH))
            UART_CO2_ISR_PATCH();
        #endif /* End UART_CO2_PSOC3_ES2*/
    }

#endif /* End UART_CO2_RX_ENABLED && (UART_CO2_RXBUFFERSIZE > UART_CO2_FIFO_LENGTH) */


#if(UART_CO2_TX_ENABLED && (UART_CO2_TXBUFFERSIZE > UART_CO2_FIFO_LENGTH))

    extern volatile uint8 UART_CO2_txBuffer[];
    extern volatile uint8 UART_CO2_txBufferRead;
    extern uint8 UART_CO2_txBufferWrite;


    /*******************************************************************************
    * Function Name: UART_CO2_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UART_CO2_txBuffer - RAM buffer pointer for transmit data from.
    *  UART_CO2_txBufferRead - cyclic index for read and transmit data 
    *     from txBuffer, increments after each transmited byte.
    *  UART_CO2_rxBufferWrite - cyclic index for write to txBuffer, 
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(UART_CO2_TXISR)
    {

        /* User code required at start of ISR */
        /* `#START UART_CO2_TXISR_START` */

        /* `#END` */

        while((UART_CO2_txBufferRead != UART_CO2_txBufferWrite) && \
             !(UART_CO2_TXSTATUS_REG & UART_CO2_TX_STS_FIFO_FULL))
        {
            /* Check pointer. */
            if(UART_CO2_txBufferRead >= UART_CO2_TXBUFFERSIZE)
            {
                UART_CO2_txBufferRead = 0u;
            }

            UART_CO2_TXDATA_REG = UART_CO2_txBuffer[UART_CO2_txBufferRead];

            /* Set next pointer. */
            UART_CO2_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START UART_CO2_TXISR_END` */

        /* `#END` */
        
        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(UART_CO2_PSOC3_ES2 && (UART_CO2_TXInternalInterrupt__ES2_PATCH))
            UART_CO2_ISR_PATCH();
        #endif /* End UART_CO2_PSOC3_ES2*/
    }

#endif /* End UART_CO2_TX_ENABLED && (UART_CO2_TXBUFFERSIZE > UART_CO2_FIFO_LENGTH) */


/* [] END OF FILE */
