/*******************************************************************************
* File Name: UART_RAD_INT.c
* Version 2.0
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "UART_RAD.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (UART_RAD_RX_ENABLED || UART_RAD_HD_ENABLED) && \
     (UART_RAD_RXBUFFERSIZE > UART_RAD_FIFO_LENGTH))

    extern volatile uint8 UART_RAD_rxBuffer[];
    extern volatile uint8 UART_RAD_rxBufferRead;
    extern volatile uint8 UART_RAD_rxBufferWrite;
    extern volatile uint8 UART_RAD_rxBufferLoopDetect;
    extern volatile uint8 UART_RAD_rxBufferOverflow;


    /*******************************************************************************
    * Function Name: UART_RAD_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UART_RAD_rxBuffer - RAM buffer pointer for save received data.
    *  UART_RAD_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     increments after each byte saved to buffer.
    *  UART_RAD_rxBufferRead - cyclic index for read from rxBuffer, 
    *     checked to detect overflow condition.
    *  UART_RAD_rxBufferOverflow - software overflow flag. Set to one
    *     when UART_RAD_rxBufferWrite index overtakes 
    *     UART_RAD_rxBufferRead index.
    *  UART_RAD_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when UART_RAD_rxBufferWrite is equal to 
    *    UART_RAD_rxBufferRead
    *
    *******************************************************************************/
    CY_ISR(UART_RAD_RXISR)
    {
        uint8 readData;

        /* User code required at start of ISR */
        /* `#START UART_RAD_RXISR_START` */

        /* `#END` */

        readData = UART_RAD_RXSTATUS_REG;

        if(readData & (UART_RAD_RX_STS_BREAK | UART_RAD_RX_STS_PAR_ERROR | \
                       UART_RAD_RX_STS_STOP_ERROR | UART_RAD_RX_STS_OVERRUN))
        {
            /* ERROR handling. */
            /* `#START UART_RAD_RXISR_ERROR` */

            /* `#END` */
        }

        while(readData & UART_RAD_RX_STS_FIFO_NOTEMPTY)
        {
            /* Check pointer. */
            if(UART_RAD_rxBufferWrite >= UART_RAD_RXBUFFERSIZE)
            {
                UART_RAD_rxBufferWrite = 0u;
                if(UART_RAD_rxBufferWrite == UART_RAD_rxBufferRead)
                {   /* Set Software Buffer status Overflow */
                    UART_RAD_rxBufferOverflow = 1u;
                }
            }

            UART_RAD_rxBuffer[UART_RAD_rxBufferWrite] = UART_RAD_RXDATA_REG;

            if(UART_RAD_rxBufferLoopDetect)
            {   /* Set Software Buffer status Overflow */
                UART_RAD_rxBufferOverflow = 1u;
            }
            /* Set next pointer. */
            UART_RAD_rxBufferWrite++;

            if(UART_RAD_rxBufferWrite == UART_RAD_rxBufferRead)
            {
                UART_RAD_rxBufferLoopDetect = 1u;
            }

            /* Check again if there is data. */
            readData = UART_RAD_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START UART_RAD_RXISR_END` */

        /* `#END` */

        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(UART_RAD_PSOC3_ES2 && (UART_RAD_RXInternalInterrupt__ES2_PATCH))
            UART_RAD_ISR_PATCH();
        #endif /* End UART_RAD_PSOC3_ES2*/
    }

#endif /* End UART_RAD_RX_ENABLED && (UART_RAD_RXBUFFERSIZE > UART_RAD_FIFO_LENGTH) */


#if(UART_RAD_TX_ENABLED && (UART_RAD_TXBUFFERSIZE > UART_RAD_FIFO_LENGTH))

    extern volatile uint8 UART_RAD_txBuffer[];
    extern volatile uint8 UART_RAD_txBufferRead;
    extern uint8 UART_RAD_txBufferWrite;


    /*******************************************************************************
    * Function Name: UART_RAD_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UART_RAD_txBuffer - RAM buffer pointer for transmit data from.
    *  UART_RAD_txBufferRead - cyclic index for read and transmit data 
    *     from txBuffer, increments after each transmited byte.
    *  UART_RAD_rxBufferWrite - cyclic index for write to txBuffer, 
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(UART_RAD_TXISR)
    {

        /* User code required at start of ISR */
        /* `#START UART_RAD_TXISR_START` */

        /* `#END` */

        while((UART_RAD_txBufferRead != UART_RAD_txBufferWrite) && \
             !(UART_RAD_TXSTATUS_REG & UART_RAD_TX_STS_FIFO_FULL))
        {
            /* Check pointer. */
            if(UART_RAD_txBufferRead >= UART_RAD_TXBUFFERSIZE)
            {
                UART_RAD_txBufferRead = 0u;
            }

            UART_RAD_TXDATA_REG = UART_RAD_txBuffer[UART_RAD_txBufferRead];

            /* Set next pointer. */
            UART_RAD_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START UART_RAD_TXISR_END` */

        /* `#END` */
        
        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(UART_RAD_PSOC3_ES2 && (UART_RAD_TXInternalInterrupt__ES2_PATCH))
            UART_RAD_ISR_PATCH();
        #endif /* End UART_RAD_PSOC3_ES2*/
    }

#endif /* End UART_RAD_TX_ENABLED && (UART_RAD_TXBUFFERSIZE > UART_RAD_FIFO_LENGTH) */


/* [] END OF FILE */
