/*******************************************************************************
* File Name: UART_SIU_INT.c
* Version 2.0
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "UART_SIU.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (UART_SIU_RX_ENABLED || UART_SIU_HD_ENABLED) && \
     (UART_SIU_RXBUFFERSIZE > UART_SIU_FIFO_LENGTH))

    extern volatile uint8 UART_SIU_rxBuffer[];
    extern volatile uint8 UART_SIU_rxBufferRead;
    extern volatile uint8 UART_SIU_rxBufferWrite;
    extern volatile uint8 UART_SIU_rxBufferLoopDetect;
    extern volatile uint8 UART_SIU_rxBufferOverflow;


    /*******************************************************************************
    * Function Name: UART_SIU_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UART_SIU_rxBuffer - RAM buffer pointer for save received data.
    *  UART_SIU_rxBufferWrite - cyclic index for write to rxBuffer, 
    *     increments after each byte saved to buffer.
    *  UART_SIU_rxBufferRead - cyclic index for read from rxBuffer, 
    *     checked to detect overflow condition.
    *  UART_SIU_rxBufferOverflow - software overflow flag. Set to one
    *     when UART_SIU_rxBufferWrite index overtakes 
    *     UART_SIU_rxBufferRead index.
    *  UART_SIU_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when UART_SIU_rxBufferWrite is equal to 
    *    UART_SIU_rxBufferRead
    *
    *******************************************************************************/
    CY_ISR(UART_SIU_RXISR)
    {
        uint8 readData;

        /* User code required at start of ISR */
        /* `#START UART_SIU_RXISR_START` */

        /* `#END` */

        readData = UART_SIU_RXSTATUS_REG;

        if(readData & (UART_SIU_RX_STS_BREAK | UART_SIU_RX_STS_PAR_ERROR | \
                       UART_SIU_RX_STS_STOP_ERROR | UART_SIU_RX_STS_OVERRUN))
        {
            /* ERROR handling. */
            /* `#START UART_SIU_RXISR_ERROR` */

            /* `#END` */
        }

        while(readData & UART_SIU_RX_STS_FIFO_NOTEMPTY)
        {
            /* Check pointer. */
            if(UART_SIU_rxBufferWrite >= UART_SIU_RXBUFFERSIZE)
            {
                UART_SIU_rxBufferWrite = 0u;
                if(UART_SIU_rxBufferWrite == UART_SIU_rxBufferRead)
                {   /* Set Software Buffer status Overflow */
                    UART_SIU_rxBufferOverflow = 1u;
                }
            }

            UART_SIU_rxBuffer[UART_SIU_rxBufferWrite] = UART_SIU_RXDATA_REG;

            if(UART_SIU_rxBufferLoopDetect)
            {   /* Set Software Buffer status Overflow */
                UART_SIU_rxBufferOverflow = 1u;
            }
            /* Set next pointer. */
            UART_SIU_rxBufferWrite++;

            if(UART_SIU_rxBufferWrite == UART_SIU_rxBufferRead)
            {
                UART_SIU_rxBufferLoopDetect = 1u;
            }

            /* Check again if there is data. */
            readData = UART_SIU_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START UART_SIU_RXISR_END` */

        /* `#END` */

        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(UART_SIU_PSOC3_ES2 && (UART_SIU_RXInternalInterrupt__ES2_PATCH))
            UART_SIU_ISR_PATCH();
        #endif /* End UART_SIU_PSOC3_ES2*/
    }

#endif /* End UART_SIU_RX_ENABLED && (UART_SIU_RXBUFFERSIZE > UART_SIU_FIFO_LENGTH) */


#if(UART_SIU_TX_ENABLED && (UART_SIU_TXBUFFERSIZE > UART_SIU_FIFO_LENGTH))

    extern volatile uint8 UART_SIU_txBuffer[];
    extern volatile uint8 UART_SIU_txBufferRead;
    extern uint8 UART_SIU_txBufferWrite;


    /*******************************************************************************
    * Function Name: UART_SIU_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UART_SIU_txBuffer - RAM buffer pointer for transmit data from.
    *  UART_SIU_txBufferRead - cyclic index for read and transmit data 
    *     from txBuffer, increments after each transmited byte.
    *  UART_SIU_rxBufferWrite - cyclic index for write to txBuffer, 
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(UART_SIU_TXISR)
    {

        /* User code required at start of ISR */
        /* `#START UART_SIU_TXISR_START` */

        /* `#END` */

        while((UART_SIU_txBufferRead != UART_SIU_txBufferWrite) && \
             !(UART_SIU_TXSTATUS_REG & UART_SIU_TX_STS_FIFO_FULL))
        {
            /* Check pointer. */
            if(UART_SIU_txBufferRead >= UART_SIU_TXBUFFERSIZE)
            {
                UART_SIU_txBufferRead = 0u;
            }

            UART_SIU_TXDATA_REG = UART_SIU_txBuffer[UART_SIU_txBufferRead];

            /* Set next pointer. */
            UART_SIU_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START UART_SIU_TXISR_END` */

        /* `#END` */
        
        /* PSoC3 ES1, ES2 RTC ISR PATCH  */
        #if(UART_SIU_PSOC3_ES2 && (UART_SIU_TXInternalInterrupt__ES2_PATCH))
            UART_SIU_ISR_PATCH();
        #endif /* End UART_SIU_PSOC3_ES2*/
    }

#endif /* End UART_SIU_TX_ENABLED && (UART_SIU_TXBUFFERSIZE > UART_SIU_FIFO_LENGTH) */


/* [] END OF FILE */
